from circle import resources

OBJECT_CLASSES = {
    resources.Ach.OBJECT_NAME: resources.Ach,
    resources.Card.OBJECT_NAME: resources.Card,
    resources.Payment.OBJECT_NAME: resources.Payment,
    resources.Payout.OBJECT_NAME: resources.Payout,
    resources.Wire.OBJECT_NAME: resources.Wire,
}
